/**********************************************************************************************************************/
/**********************************************************************************************************************/
/***    This program is associated to the paper                                                             ***********/
/***   "Pure range-only SLAM with indistinguishable landmarks; a constraint programming approach"           ***********/
/***   See    http://www.ensta-bretagne.fr/jaulin/mslam.html                                                ***********/
/**********************************************************************************************************************/
/**********************************************************************************************************************/


#include "mainwindow.h"
#include "ui_mainwindow.h"

double xmin,xmax,ymin,ymax;
double epsilon;


MainWindow::MainWindow(QWidget *parent) :  QMainWindow(parent),
      ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    xmin=-2;xmax=2;
    ymin=-3;ymax=3;
     ui->graphicsView->setInteractive(true);
    R=new repere(this,ui->graphicsView,xmin,xmax,ymin,ymax);
}

void MainWindow::Init()
{ epsilon=ui->EpsilonSpinBox->value();
}

MainWindow::~MainWindow()
{
    delete ui;
}

iboolean Inside (box& X)
{   interval F1=Sqr(X[1])+Sqr(X[2])+Sin(X[1]+X[2]);
    iboolean T1=In(F1,interval(4,9));
    return (T1);
};

void MainWindow::on_ButtonStart_clicked()
{     Init();
      vector <interval> X,X0;
      double tmin=xmin;
      double tmax=xmax;
      double dt=0.1;
      for (double t=tmin;t<tmax;t=t+dt)    // we first build the initial tube
      {   interval Xk(-2*sqrt(1-cos(1.8*t)*sin(2.7*t)),2*sqrt(1+cos(1.2*t)*sin(2.1*t)));
          X.push_back(Xk);
      }      
      X0=X;

      int kmax=X.size();
      double deltat=1.0;

      // we paint the tube
      for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,X[k].inf,X[k].sup,QPen(Qt::black),QBrush(Qt::darkGray));   R->Save("paving1_X");

      vector <interval> Y;
      tube_shift (Y,X,deltat,dt);   // we translate the tube by deltat

      R->Clean();
      for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,X0[k].inf,X0[k].sup,QPen(Qt::gray),QBrush(Qt::lightGray));
      for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,Y[k].inf,Y[k].sup,QPen(Qt::black),QBrush(Qt::darkGray)); R->Save("paving2_Y");


      tube_Cegal (Y,X);     // we intersect the two tubes, i.e., X:=X \cap Y and Y:=Y \cap X

      tube_Cshift (Y,X,deltat,dt);  //  we contract the two tubes X and Y, thanks to the constraint Y(t)=X(t+deltat)
      R->Clean();
      for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,X0[k].inf,X0[k].sup,QPen(Qt::gray),QBrush(Qt::lightGray));
      for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,X[k].inf,X[k].sup,QPen(Qt::black),QBrush(Qt::darkGray));  R->Save("paving3_XinterX");


     for (int i=1;i<=5;i++)   // we contract up to the fixed point
      {    tube_Cshift (Y,X,deltat,dt);
           R->Clean();
           for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,X0[k].inf,X0[k].sup,QPen(Qt::gray),QBrush(Qt::lightGray));
           for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,X[k].inf,X[k].sup,QPen(Qt::black),QBrush(Qt::darkGray)); R->Save("paving3bis"+QString::number(10.0+i));
      }

     R->Clean();
     for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,X0[k].inf,X0[k].sup,QPen(Qt::gray),QBrush(Qt::lightGray));
     for (int k=0;k<kmax;k++)  R->DrawBox(tmin+k*dt,tmin+(k+1)*dt,X[k].inf,X[k].sup,QPen(Qt::black),QBrush(Qt::darkGray)); R->Save("paving4_fixPoint");


}
